using System;

using Syncfusion.Collections.BinaryTree;
using Syncfusion.Grouping;

namespace CustomSummary
{
    public class TimeSpanSummary : SummaryBase
    {
        public static readonly TimeSpanSummary Empty = new TimeSpanSummary(TimeSpan.Zero);

        private readonly TimeSpan _total;

        public TimeSpanSummary(TimeSpan total)
        {
            _total = total;
        }

        public TimeSpanSummary Combine(TimeSpanSummary other)
        {
            // Summary objects are immutable. That means properties cannot be modified for an 
            // existing object. Instead every time a change is made a new object must be created (just like 
            // System.String). 
            //
            // This allows following optimization: return existing summary object if either one of the values is 0. -->
            if (other.Total.Ticks == 0)
            {
                return this;
            }
            else if (Total.Ticks == 0)
            {
                return other;
            }
            // <-- end of optimization
            else
            {
                return new TimeSpanSummary(Total + other.Total);
            }
        }

        // To return the custom calculated value for the summary field
        public override SummaryBase Combine(SummaryBase other)
        {
            return Combine((TimeSpanSummary)other);
        }

        public static ITreeTableSummary CreateSummaryMethod(SummaryDescriptor sd, Record record)
        {
            object obj = sd.GetValue(record);
            bool isNull = ((obj == null) || obj is DBNull);
            if (isNull)
            {
                return Empty;
            }
            else if(obj is TimeSpan)
            {
                TimeSpan ts = (TimeSpan)obj;
                double val = ts.TotalMinutes;
                return new TimeSpanSummary(ts);
            }
            else
            {
                //double val = Convert.ToDouble(obj);
                //return new TotalSummary(val);
            }
            return new CountSummary(0);
        }

        public TimeSpan Total
        {
            get
            {
                return _total;
            }
        }
    }
}